@@ -3,7 +3,9 @@ import os
 def read(filePath: str):
     if os.path.isabs(filePath):
         f = open(file=filePath)
-        return f.read()
+        content: str = f.read()
+        f.close()
+        return content
     else:
         raise FileNotFoundError("Please specify absolute path")
 
